@@ -241,7 +241,7 @@ setMethod(
 #'
 #' Return a list (or data.frame) of raw item and group level coefficients. Note that while
 #' the output to the console is rounded to three digits, the returned list of objects is not.
-#' Hence, elements from \code{cfs <- coef(mod); cfs[[1]]} will contain the unrounded results (useful
+#' Hence, elements from \code{cfs <- coef(mod); cfs[[1]]} will contain the non-rounded results (useful
 #' for simulations).
 #'
 #' @param object an object of class \code{SingleGroupClass},
@@ -249,7 +249,8 @@ setMethod(
 #' @param CI the amount of converged used to compute confidence intervals; default is
 #'   95 percent confidence intervals
 #' @param IRTpars logical; convert slope intercept parameters into traditional IRT parameters?
-#'   Only applicable to unidimensional models. If a suitable ACOV estimate was computed in the fitted
+#'   Only applicable to unidimensional models or models with simple structure (i.e., only one non-zero slope).
+#'   If a suitable ACOV estimate was computed in the fitted
 #'   model, and \code{printSE = FALSE}, then suitable CIs will be included based on the delta
 #'   method (where applicable)
 #' @param rotate see \code{summary} method for details. The default rotation is \code{'none'}
@@ -331,13 +332,19 @@ setMethod(
             object@ParObjects$pars[[J + 1]]@par[-c(1:nfact)] <- so$fcor[lower.tri(so$fcor, TRUE)]
         }
         if(IRTpars){
-            if(object@Model$nfact > 1L)
-                stop('traditional parameterization is only available for unidimensional models',
-                     call.=FALSE)
+            if(object@Model$nfact > 1L){
+                apars <- lapply(object@ParObjects$pars[-(extract.mirt(object, 'nitems') + 1L)],
+                                function(x) x@par[1L:object@Model$nfact])
+                is_ss <- sapply(apars, function(x) sum(x != 0) == 1L)
+                if(!any(is_ss))
+                    stop(c('traditional parameterization is only available for unidimensional ',
+                           'models or models with simple structure patterns'), call.=FALSE)
+            }
             vcov <- vcov(object)
             for(i in 1L:J){
                 if(class(object@ParObjects$pars[[i]]) %in% c('gpcmIRT')) next
-                object@ParObjects$pars[[i]] <- mirt2traditional(object@ParObjects$pars[[i]], vcov=vcov)
+                object@ParObjects$pars[[i]] <- mirt2traditional(object@ParObjects$pars[[i]],
+                                                                vcov=vcov, nfact=object@Model$nfact)
 
             }
         }
@@ -713,7 +720,7 @@ setMethod(
 #' mod <- mirt(dat, 1)
 #'
 #' # JSI executed in parallel over multiple cores
-#' mirtCluster()
+#' if(interactive()) mirtCluster()
 #' residuals(mod, type = 'JSI')
 #'
 #' }
@@ -1003,6 +1010,7 @@ setMethod(
 #'     \item{\code{'itemscore'}}{item scoring traceline plots}
 #'     \item{\code{'score'}}{expected total score surface}
 #'     \item{\code{'scorecontour'}}{expected total score contour plot}
+#'     \item{\code{'posteriorTheta'}}{posterior for the latent trait distribution}
 #'     \item{\code{'EAPsum'}}{compares sum-scores to the expected values based
 #'       on the EAP for sum-scores method (see \code{\link{fscores}})}
 #'   }
@@ -1068,6 +1076,7 @@ setMethod(
 #' plot(x, type = 'infotrace', facet_items = FALSE)
 #' plot(x, type = 'infoSE')
 #' plot(x, type = 'rxx')
+#' plot(x, type = 'posteriorTheta')
 #'
 #' # confidence interval plots when information matrix computed
 #' plot(x)
@@ -1081,6 +1090,12 @@ setMethod(
 #' plt <- plot(x, type = 'trace')
 #' direct.label(plt, 'top.points')
 #'
+#' # additional modifications can be made via update().
+#' # See ?update.trellis for further documentation
+#' plt
+#' update(plt, ylab = expression(Prob(theta)),
+#'             main = "Item Traceline Functions") # ylab/main changed
+#'
 #' set.seed(1234)
 #' group <- sample(c('g1','g2'), nrow(Science), TRUE)
 #' x2 <- multipleGroup(Science, 1, group)
@@ -1113,7 +1128,7 @@ setMethod(
         dots <- list(...)
         if(!(type %in% c('info', 'SE', 'infoSE', 'rxx', 'trace', 'score', 'itemscore',
                        'infocontour', 'infotrace', 'scorecontour', 'empiricalhist', 'Davidian',
-                       'EAPsum')))
+                       'EAPsum', 'posteriorTheta')))
             stop('type supplied is not supported')
         if (any(degrees > 90 | degrees < 0))
             stop('Improper angle specified. Must be between 0 and 90.', call.=FALSE)
@@ -1620,94 +1635,115 @@ setMethod(
                               xlab = expression(theta), ylab = 'Density',
                               type = 'b', main = main,
                               par.strip.text=par.strip.text, par.settings=par.settings, ...))
-            } else {
+            } else if(type == 'posteriorTheta'){
+                if(is.null(main))
+                    main <- 'Posterior Distribution of Latent Trait'
+                plt <- extract.mirt(x, 'thetaPosterior')[[1]]
+                return(xyplot(posterior ~ Theta, plt,
+                              xlab = expression(theta), ylab = 'Density',
+                              type = 'b', main = main,
+                              par.strip.text=par.strip.text, par.settings=par.settings, ...))
+
+            }else {
                 stop('plot not supported for unidimensional models', call.=FALSE)
             }
         }
     }
 )
 
-mirt2traditional <- function(x, vcov){
+mirt2traditional <- function(x, vcov, nfact){
     cls <- class(x)
     opar <- par <- x@par
+    which.a <- which(x@par[1L:nfact] != 0)
+    if(length(which.a) != 1L) return(x)
+    a.nms <- if(nfact == 1L) 'a' else paste0('a', 1L:nfact)
     if(cls != 'GroupPars')
         ncat <- x@ncat
     if(cls == 'dich'){
-        fns <- vector('list', 4L)
-        fns[[2]] <- function(par, index, opar){
-            if(index == 2L){
-                opar[1L:2L] <- par
-                ret <- -opar[2L]/opar[1L]
+        fns <- vector('list', nfact + 3L)
+        fns[[nfact+1L]] <- function(par, index, opar){
+            if(index == (nfact + 1L)){
+                opar[c(which.a, nfact + 1L)] <- par
+                ret <- -opar[nfact + 1L]/opar[which.a]
             }
             ret
         }
-        fns[[3]] <- function(par, index, opar){
-            if(index == 3L)
+        fns[[nfact+2L]] <- function(par, index, opar){
+            if(index == nfact + 2L)
                 ret <- plogis(par)
             ret
         }
-        fns[[4]] <- function(par, index, opar){
-            if(index == 4L)
+        fns[[nfact+3L]] <- function(par, index, opar){
+            if(index == nfact + 3L)
                 ret <- plogis(par)
             ret
         }
-        delta_index <- list(NA, 1L:2L, 3L, 4L)
-        par[2] <- -par[2]/par[1]
-        par[3] <- plogis(par[3])
-        par[4] <- plogis(par[4])
-        names(par) <- c('a', 'b', 'g', 'u')
+        delta_index <- c(as.list(rep(NA, nfact)),
+                         list(c(which.a, nfact + 1L),
+                              nfact + 2L, nfact+3L))
+        par[nfact + 1L] <- -par[nfact + 1L]/par[which.a]
+        par[nfact + 2L] <- plogis(par[nfact + 2L])
+        par[nfact + 3L] <- plogis(par[nfact + 3L])
+        names(par) <- c(a.nms, 'b', 'g', 'u')
     } else if(cls == 'graded'){
-        fns <- vector('list', ncat+1L)
-        for(i in 2L:ncat){
-            fns[[i]] <- function(par, index, opar){
-                if(index > 1L){
-                    opar[c(1L, index)] <- par
-                    ret <- -opar[index]/opar[1L]
+        fns <- vector('list', ncat + nfact-1L)
+        for(i in 2L:ncat - 1L){
+            fns[[i + nfact]] <- function(par, index, opar){
+                if(index > nfact){
+                    opar[c(which.a, index)] <- par
+                    ret <- -opar[index]/opar[which.a]
                 }
                 ret
             }
         }
-        delta_index <- vector('list', ncat)
-        delta_index[[1L]] <- NA
-        for(i in 2:ncat){
-            par[i] <- -par[i]/par[1]
-            delta_index[[i]] <- c(1L, i)
+        delta_index <- vector('list', ncat + nfact - 1L)
+        for(i in 1:nfact)
+            delta_index[[i]] <- NA
+        for(i in 2:ncat-1L){
+            par[i+nfact] <- -par[i+nfact]/par[which.a]
+            delta_index[[i+nfact]] <- c(which.a, i+nfact)
         }
-        names(par) <- c('a', paste0('b', 1:(length(par)-1)))
+        names(par) <- c(a.nms, paste0('b', 2:ncat-1L))
     } else if(cls == 'gpcm'){
-        fns <- vector('list', ncat+1L)
-        for(i in 2L:ncat){
-            fns[[i]] <- function(par, index, opar){
-                if(index > 1L){
-                    if(index == 2L) opar[c(1, ncat + 3)] <- par
-                    else opar[c(1, ncat + index, ncat + index + 1)] <- par
+        fns <- vector('list', ncat+nfact)
+        for(i in 2L:ncat-1L){
+            fns[[i+nfact]] <- function(par, index, opar){
+                if(index > nfact){
+                    if(index == (nfact+1))
+                        opar[c(which.a, ncat + nfact + 2L)] <- par
+                    else opar[c(which.a, ncat + index + nfact - 1L,
+                                ncat + index + nfact)] <- par
                     par <- opar
-                    ds <- par[-1]/par[1]
+                    ds <- par[(nfact+1):length(par)]/par[which.a]
                     ds <- ds[-seq_len(ncat)]
                     newd <- numeric(length(ds)-1L)
                     for(i in 2:length(ds))
                         newd[i-1L] <- -(ds[i] - ds[i-1L])
-                    ret <- c(par[1], newd)
+                    ret <- c(par[1:nfact], newd)
                     ret <- ret[index]
                 }
                 ret
             }
         }
-        delta_index <- vector('list', ncat)
-        delta_index[[1L]] <- NA
-        ds <- par[-1]/par[1]
+        delta_index <- vector('list', ncat+nfact-1L)
+        for(i in 1:nfact)
+            delta_index[[i]] <- NA
+        ds <- par[(nfact+1):length(par)]/par[which.a]
         ds <- ds[-seq_len(ncat)]
         newd <- numeric(length(ds)-1L)
+        for(i in 2:length(ds))
+            newd[i-1L] <- -(ds[i] - ds[i-1L])
         tmp <- rbind(1:(ncat-1), 2:ncat)
         for(i in 2:length(ds)){
             newd[i-1L] <- -(ds[i] - ds[i-1L])
-            delta_index[[i]] <- c(1L, tmp[,i-1L] + ncat + 1L)
+            delta_index[[i+nfact-1L]] <- c(which.a, tmp[,i-1L] + ncat + nfact)
         }
-        delta_index[[2L]] <- c(1L, ncat + 3L)
-        par <- c(par[1], newd)
-        names(par) <- c('a', paste0('b', 1:length(newd)))
-        x@est <- x@est[c(1, (ncat+3L):length(x@est))]
+        delta_index[[nfact+1L]] <- c(which.a, ncat + nfact + 2L)
+        par <- c(x@par[1:nfact], newd)
+        names(par) <- c(a.nms, paste0('b', 1:length(newd)))
+        x@est <- x@est[c(1:nfact, (ncat+nfact+2L):length(x@est))]
     } else if(cls == 'nominal'){
+        if(nfact > 1L) return(x)
         fns <- vector('list', ncat*2)
         for(i in 2L:length(par)-1L){
             fns[[i]] <- function(par, index, opar){
@@ -1731,10 +1767,11 @@ mirt2traditional <- function(x, vcov){
         ds <- par[(ncat+2):length(par)]
         ds <- ds - mean(ds)
         par <- c(as, ds)
-        names(par) <- c(paste0('a', 1:ncat), paste0('c', 1:ncat))
+        names(par) <- c(a.nms, paste0('c', 1:ncat))
         x@est <- rep(TRUE, ncat*2)
         x@SEpar <- rep(as.numeric(NA), ncat*2)
     } else if(cls == 'nestlogit'){
+        if(nfact > 1L) return(x)
         fns <- vector('list', ncat*2 + 4)
         fns[[2]] <- function(par, index, opar){
             if(index == 2L){
@@ -1776,7 +1813,7 @@ mirt2traditional <- function(x, vcov){
         par1[2] <- -par1[2]/par1[1]
         par1[3] <- plogis(par1[3])
         par1[4] <- plogis(par1[4])
-        names(par1) <- c('a', 'b', 'g', 'u')
+        names(par1) <- c(a.nms, 'b', 'g', 'u')
         par2 <- par[5:length(par)]
         as <- par2[1:(ncat-1)]
         as <- as - mean(as)
@@ -1821,7 +1858,7 @@ mirt2traditional <- function(x, vcov){
 #' This is a helper function for users who have previously available traditional/classical
 #' IRT parameters and want to know the equivalent slope-intercept translation used in \code{mirt}.
 #' Note that this function assumes that the supplied models are unidimensional by definition (i.e.,
-#' will have only one slope/discrimination). If there is no supported slope-interecept transformation
+#' will have only one slope/discrimination). If there is no supported slope-intercept transformation
 #' available then the original vector of parameters will be returned by default.
 #'
 #' Supported class transformations for the \code{cls} input are:
