@@ -347,7 +347,7 @@ itemfit <- function(x, fit_stats = 'S_X2',
             Theta <- thetaComb(seq(ETrange[1L], ETrange[2L], length.out=ETpoints),
                                nfact=nfact)
             dat <- extract.mirt(mod, 'data')
-            Emod <- mirt(dat, extract.mirt(mod, 'model'), itemtype=itemtype,
+            Emod <- mirt(dat, nfact, itemtype=itemtype,
                          pars=sv, verbose=FALSE,
                          technical=list(storeEtable=TRUE, customTheta=Theta), ...)
             Etable <- Emod@Internals$Etable[[1]]$r1
