@@ -9,7 +9,8 @@
 #'
 #' @aliases mirtCluster
 #' @param spec input that is passed to \code{parallel::makeCluster()}. If no input is given the
-#'   maximum number of available local cores will be used. Setting this to NULL will skip a new definition (allows \code{omp_threads} to be used independently)
+#'   maximum number of available local cores minus 1 will be used.
+#'   Setting this to NULL will skip a new definition (allows \code{omp_threads} to be used independently)
 #' @param omp_threads number of OpenMP threads to use (currently applies to E-step computations only).
 #'   Not used when argument input is missing
 #' @param ... additional arguments to pass to \code{parallel::makeCluster}
@@ -51,7 +52,7 @@ mirtCluster <- function(spec, omp_threads, remove = FALSE, ...){
         return(invisible(NULL))
     if(requireNamespace("parallel", quietly = TRUE)){
         if(missing(spec))
-            spec <- parallel::detectCores()
+            spec <- parallel::detectCores() - 1L
         if(remove){
             if(is.null(.mirtClusterEnv$MIRTCLUSTER)){
                 message('There is no visible mirtCluster() definition')
