@@ -83,7 +83,7 @@ personfit <- function(x, method = 'EAP', Theta = NULL, stats.only = TRUE, ...){
         stop('Discrete latent structures not yet supported', call.=FALSE)
     if(is(x, 'MixtureClass'))
         stop('Mixture latent structures not yet supported', call.=FALSE)
-    if(any(is.na(x@Data$data)))
+    if(any(is.na(x@Data$data)) && is.null(attr(x, 'inoutfitreturn')))
         stop('Fit statistics cannot be computed when there are missing data.', call.=FALSE)
     if(is(x, 'MultipleGroupClass')){
         ret <- vector('list', x@Data$ngroups)
